@@ -1,19 +1,23 @@
-# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import os
 import socket
@@ -24,23 +28,24 @@ import StringIO
 import traceback
 
 from workbench.utils import Version
-from wb_admin_ssh import SSHDownException
-from workbench.db_utils import MySQLConnection, MySQLError, QueryError, strip_password
+from workbench.db_utils import MySQLConnection, MySQLError, QueryError, strip_password, escape_sql_string
 
 from wb_common import OperationCancelledError, Users, PermissionDeniedError, InvalidPasswordError, SSHFingerprintNewError
 
 from wb_server_control import PasswordHandler, ServerControlShell, ServerControlWMI
-from wb_server_management import ServerManagementHelper, SSH, wbaOS
+from wb_server_management import ServerManagementHelper, wbaOS
 
 from workbench.notifications import nc
 
 import grt
 import mforms
 
-from workbench.log import log_info, log_warning, log_error, log_debug
+from workbench.log import log_info, log_warning, log_error, log_debug, log_debug2, log_debug3
+from wb_common import to_unicode
 
 MYSQL_ERR_ACCESS_DENIED = 1045
 MYSQL_ERR_PASSWORD_EXPIRED = 1820
+MYSQL_ERR_OFFLINE_MODE = 3032
 
 #===============================================================================
 #
@@ -48,7 +53,7 @@ MYSQL_ERR_PASSWORD_EXPIRED = 1820
 # Handling object should have corresponding method if the object wants to receive
 # events. For event shutdown method name must be shutdown_event
 class EventManager(object):
-    valid_events = ['server_started_event', 'server_stopped_event', 'shutdown_event']
+    valid_events = ['server_offline_event', 'server_started_event', 'server_stopped_event', 'shutdown_event', 'firewall_installed_event', 'firewall_removed_event']
 
     def __init__(self):
         self.events = {}
@@ -67,9 +72,8 @@ class EventManager(object):
                 self.events[event_name] = handlers_list
 
             handlers_list.append(handler)
-            log_debug("Added " + handler.__class__.__name__ + " for event " + event_name + '\n')
+            log_debug3("Added " + handler.__class__.__name__ + " for event " + event_name + '\n')
         else:
-            print "Error! ", handler.__class__, " does not have method", event_name
             log_error(handler.__class__.__name__ + " does not have method " + event_name + '\n')
 
     def defer_events(self):
@@ -85,19 +89,17 @@ class EventManager(object):
         if self.defer:
             self.deferred_events.append(name)
             return
-
         name += "_event"
         if name not in self.valid_events:
-            print "EventManager: invalid event", name
             log_error('EventManager: invalid event: ' + name + '\n')
         elif name in self.events:
-            log_debug("Found event " + name + " in list" + '\n')
+            log_debug3("Found event " + name + " in list" + '\n')
             for obj in self.events[name]:
                 if hasattr(obj, name):
-                    log_debug("Passing event " + name + " to " + obj.__class__.__name__ + '\n')
+                    log_debug3("Passing event " + name + " to " + obj.__class__.__name__ + '\n')
                     getattr(obj, name)()
         else:
-            log_debug("Found valid but unrequested event " + name + " in list" + '\n')
+            log_debug3("Found valid but unrequested event " + name + " in list" + '\n')
 
 #===============================================================================
 #import thread
@@ -106,7 +108,7 @@ class SQLQueryExecutor(object):
     mtx     = None
 
     def __init__(self, dbconn):
-        log_debug("Constructing SQL query runner, dbconn (" + repr(dbconn) + ')\n')
+        log_debug2("Constructing SQL query runner, dbconn (" + repr(dbconn) + ')\n')
         self.mtx = threading.Lock()
         self.dbconn = dbconn
 
@@ -128,7 +130,7 @@ class SQLQueryExecutor(object):
         self.mtx.acquire()
         try:
             if self.is_connected():
-                log_debug("Executing query %s\n" % strip_password(query))
+                log_debug3("Executing query %s\n" % strip_password(query))
                 result = self.dbconn.executeQuery(query)
         finally:
             self.mtx.release()
@@ -139,7 +141,7 @@ class SQLQueryExecutor(object):
         self.mtx.acquire()
         try:
             if self.is_connected():
-                log_debug("Executing query multi result %s\n" % strip_password(query))
+                log_debug3("Executing query multi result %s\n" % strip_password(query))
                 result = self.dbconn.executeQueryMultiResult(query)
         finally:
             self.mtx.release()
@@ -150,7 +152,7 @@ class SQLQueryExecutor(object):
         self.mtx.acquire()
         try:
             if self.is_connected():
-                log_debug("Executing statement %s\n" % strip_password(query))
+                log_debug3("Executing statement %s\n" % strip_password(query))
                 result = self.dbconn.execute(query)
         finally:
             self.mtx.release()
@@ -165,7 +167,6 @@ class WbAdminControl(object):
     server_helper = None # Instance of ServerManagementHelper for doing file and process operations on target server
     server_control = None # Instance of ServerControlBase to do start/stop/status of MySQL server
 
-    ssh = None
     sql = None
 
     worker_thread = None
@@ -194,10 +195,16 @@ class WbAdminControl(object):
 
         self.add_me_for_event("server_started", self)
         self.add_me_for_event("server_stopped", self)
+        self.add_me_for_event("server_offline", self)
 
         self.editor = editor
         if editor:
-            self.last_known_server_running_status = ("running" if editor.isConnected else "stopped", None) # (status, timestamp)
+            if editor.isConnected == 1:
+                self.last_known_server_running_status = ("running", None) # (status, timestamp)
+            elif editor.isConnected == -1:
+                self.last_known_server_running_status = ("offline", None) # (status, timestamp)
+            else:
+                self.last_known_server_running_status = ("stopped", None) # (status, timestamp)
         else:
             self.last_known_server_running_status = ("unknown", None)
 
@@ -208,11 +215,14 @@ class WbAdminControl(object):
 
         # Sets the default logging callback
         self.log_cb = self.raw_log
-
+        self.sshBridge = None
 
     def raw_log(self, data):
         log_info(data + "\n")
 
+    @property
+    def ssh(self):
+        return self.sshBridge
 
     @property
     def admin_access_available(self):
@@ -243,45 +253,32 @@ class WbAdminControl(object):
         """Make sure we have access to the instance for admin (for config file, start/stop etc)"""
         if self.server_control or not self.server_profile.admin_enabled:
             return
+        
         if self.server_profile.uses_ssh:
-            try:
-                while True:
-                    try:
-                        self.ssh = SSH(self.server_profile, self.password_handler)
-                        break
-                    except SSHFingerprintNewError, exc:
-                        if self._confirm_server_fingerprint(self.server_profile.ssh_hostname, self.server_profile.ssh_port, exc):
-                            continue;
-                        else:
-                            raise OperationCancelledError("user cancel")
-                    
-            except OperationCancelledError:
-                self.ssh = None
-                raise OperationCancelledError("SSH connection cancelled")
-
-            except SSHDownException, e:
-                log_error("SSHDownException: %s\n" % traceback.format_exc())
-                self.ssh = None
-                if self.sql_enabled:
-                    if mforms.Utilities.show_warning('SSH connection failed',
-                                                     "Check your SSH connection settings and whether the SSH server is up.\nYou may continue anyway, but some functionality will be unavailable.\nError: %s" % e,
-                                                     "Continue", "Cancel", "") != mforms.ResultOk:
+            if self.editor is not None:
+                if self.editor.sshConnection.isConnected() == 0:
+                    if self.editor.sshConnection.connect() != 0:
                         raise OperationCancelledError("Could not connect to SSH server")
-                else:
-                    mforms.Utilities.show_warning('SSH connection failed',
-                                                  "Check your SSH connection settings and whether the SSH server is up.\nError: %s" % e, "OK", "", "")
+                self.sshBridge = self.editor.sshConnection
+                self.server_helper = ServerManagementHelper(self.server_profile, self.sshBridge)
+            else:
+                ssh = grt.modules.Workbench.createSSHSession(self.server_profile.get_settings_object())
+                if ssh.connect() != 0:
+                    raise OperationCancelledError("Could not connect to SSH server")
+                self.sshBridge = ssh 
+                self.server_helper = ServerManagementHelper(self.server_profile, self.sshBridge)
         else:
-            self.ssh = None
-        # init server management helper (for doing remote file operations, process management etc)
-        self.server_helper = ServerManagementHelper(self.server_profile, self.ssh)
+            self.server_helper = ServerManagementHelper(self.server_profile, None)
+        
         if self.server_helper:
             self.server_profile.expanded_config_file_path = self.server_helper.shell.expand_path_variables(self.server_profile.config_file_path)
             self.query_server_installation_info()
+        
         # detect the exact type of OS we're managing
         os_info = self.detect_operating_system_version()
         if os_info:
             os_type, os_name, os_variant, os_version = os_info
-            log_info("Target OS detection returned: os_type=%s, os_name=%s, os_variant=%s, os_version=%s\n" % (os_type, os_name, os_variant, os_version))
+            log_debug("Target OS detection returned: os_type=%s, os_name=%s, os_variant=%s, os_version=%s\n" % (os_type, os_name, os_variant, os_version))
             self.server_profile.set_os_version_info(os_type or "", os_name or "", os_variant or "", os_version or "")
         else:
             log_warning("Could not detect target OS details\n")
@@ -337,25 +334,36 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
     def shutdown(self):
         self.events.event('shutdown')
         self.running = False
-        self.disconnect_sql()
-        if self.ssh:
-            self.ssh.close()
+        if self.worker_thread:
+            self.worker_thread.join()
 
+        self.server_helper = None
+        self.disconnect_sql()
+        if self.sshBridge and self.sshBridge.isConnected() == 1:
+            self.sshBridge.disconnect()
+            self.sshBridge = None
 
     #---------------------------------------------------------------------------
     def force_check_server_state(self, verbose = False):
         # Check the current state of the server and cause the SQL Editor to reconnect/update if the state
         # changed. Returns None if no state change was detected or the new state if it did change.
-        
+        old_state = self.last_known_server_running_status[0]
         new_state = self.is_server_running(verbose=verbose, force_hard_check=True)
         log_debug("Force check server state returned %s\n" % new_state)
-        if new_state != self.last_known_server_running_status[0]:
-            info = { "state" : 1 if new_state == "running" else 0, "connection" : self.server_profile.db_connection_params }
+        if new_state != old_state:
+            info = { "state" : -1, "connection" : self.server_profile.db_connection_params }
+            if new_state == "running":
+                info['state'] = 1 
+            elif new_state == "offline":
+                info['state'] = -1;
+            else:
+                info['state'] = 0;
+            
             # this will notify the rest of the App that the server state has changed, giving them a chance
             # to reconnect or formally disconnect
             nc.send("GRNServerStateChanged", self.editor, info)
             
-            if new_state == "stopped":
+            if new_state == "stopped" or new_state == "offline":
                 self.server_variables = {}
                 self.status_variables_time = None
                 self.status_variables = {}
@@ -369,7 +377,6 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
         # Check recent information from query runs. If last query (not older than 4 secs)
         # was performed successfully, then we imply that server is up.
         ret = "unknown"
-
         status, stime = self.last_known_server_running_status
         if not force_hard_check:
             ret = status
@@ -379,8 +386,10 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
                 # do a ping on the connection to make sure it's still up
                 if self.sql_ping():
                     self.log_cb("MySQL server is currently running")
-                    ret = "running"
-            if ret != "running":
+                    self.query_server_info() #query_server_info do update server status
+                    status, stime = self.last_known_server_running_status
+                    ret = status
+            if ret not in ("running", "offline"):
                 self.log_cb("Trying to connect to MySQL...")
 
                 conn = grt.modules.DbMySQLQuery.openConnection(self.server_profile.db_connection_params)
@@ -393,10 +402,15 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
                     self.log_cb("Connection succeeded")
                 else:
                     self.log_cb("%s (%i)" % (err, code))
+
                 if code in (2002, 2003, 2013):
                     ret = "stopped"
                     self.log_cb("Assuming server is not running")
+                elif code == MYSQL_ERR_OFFLINE_MODE:
+                    ret = "offline"
+                    self.log_cb("Server is in offline mode")
                 else:
+                    self.query_server_info()
                     ret = "running"
                     self.log_cb("Assuming server is running")
 
@@ -470,10 +484,19 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             log_error("Error connecting to MySQL: %s\n" % e)
             mforms.Utilities.show_error("Connect Error", "Could not connect to MySQL: %s" % e, "OK", "", "")
 
+        if not self.worker_thread.is_alive() and not self.test_only:
+            # start status variable check thread
+            self.worker_thread = threading.Thread(target=self.server_polling_thread)
+            self.worker_thread.start()
+
     #---------------------------------------------------------------------------
     def server_stopped_event(self):
         self.disconnect_sql()
         self.last_known_server_running_status = ("stopped", time.time())
+        
+    #---------------------------------------------------------------------------
+    def server_offline_event(self):
+        self.last_known_server_running_status = ("offline", time.time())
 
     #---------------------------------------------------------------------------
     """
@@ -483,7 +506,10 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
     """
     def sql_status_callback(self, code, error, connect_info):
         if code == 0:
-            self.last_known_server_running_status = ("running", time.time())
+            if "offline_mode" in self.server_variables and self.server_variables["offline_mode"] == "ON":
+                self.last_known_server_running_status = ("offline", time.time()) 
+            else:
+                self.last_known_server_running_status = ("running", time.time())
 
 
     #---------------------------------------------------------------------------
@@ -496,7 +522,7 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             try:
                 connection.connect()
             except grt.UserInterrupt:
-                log_info("Cancelled connection\n")
+                log_debug("Cancelled connection\n")
                 return
 
             self.sql = SQLQueryExecutor(connection)
@@ -507,7 +533,7 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             else:
                 log_error("Failed to connect to MySQL server\n")
         else:
-            log_info("Already connected to MySQL server\n")
+            log_debug("Already connected to MySQL server\n")
 
     #---------------------------------------------------------------------------
     def disconnect_sql(self):
@@ -537,11 +563,12 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             mforms.Utilities.driver_shutdown()
             return None
 
-        log_info("Monitoring thread running...\n")
+        log_debug("Monitoring thread running...\n")
         time.sleep(self.status_variable_poll_interval)
         try:
             # runs in a separate thread to fetch status variables
             while self.running:
+                log_debug3("Poll server status\n")
                 variables = {}
                 result = self.poll_connection.executeQuery("SHOW GLOBAL STATUS")
                 while result and result.nextRow():
@@ -553,7 +580,7 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
         except QueryError:
             log_error("Error in monitoring thread: %s\n" % traceback.format_exc())
 
-        log_info("Monitoring thread done.\n")
+        log_debug("Monitoring thread done.\n")
         self.poll_connection.disconnect()
         self.poll_connection = None
         mforms.Utilities.driver_shutdown()
@@ -592,14 +619,15 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             try:
                 ret = self.sql.exec_query(q)
             except QueryError, e:
-                log_warning("Error executing query %s: %s\n"%(q, strip_password(str(e))))
+                log = u"Error executing query %s: %s\n" % (q, strip_password(to_unicode(e.message)))
+                log_warning(log)
                 if auto_reconnect and e.is_connection_error():
                     log_warning("exec_query: Loss of connection to mysql server was detected.\n")
                     self.handle_sql_disconnection(e)
                 else: # if exception is not handled, give a chance to the caller do it
                     raise e
         else:
-            log_info("sql connection is down\n")
+            log_debug("sql connection is down\n")
 
         return ret
 
@@ -609,14 +637,14 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             try:
                 ret = self.sql.exec_query_multi_result(q)
             except QueryError, e:
-                log_warning("Error executing query multi result %s: %s\n"%(q, strip_password(str(e))))
+                log_warning("Error executing query multi result %s: %s\n"%(q, strip_password(e.message)))
                 if auto_reconnect and e.is_connection_error():
                     log_warning("exec_query_multi_result: Loss of connection to mysql server was detected.\n")
                     self.handle_sql_disconnection(e)
                 else: # if exception is not handled, give a chance to the caller do it
                     raise e
         else:
-            log_info("sql connection is down\n")
+            log_debug("sql connection is down\n")
 
         return ret
 
@@ -627,14 +655,14 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
                 cnt = self.sql.updateCount()
                 return ret, cnt
             except QueryError, e:
-                log_warning("Error executing SQL %s: %s\n"%(strip_password(q), strip_password(str(e))))
+                log_warning("Error executing SQL %s: %s\n"%(strip_password(q), strip_password(e.message)))
                 if auto_reconnect and e.is_connection_error():
                     log_warning("exec_sql: Loss of connection to mysql server was detected.\n")
                     self.handle_sql_disconnection(e)
                 else:
                     raise e
         else:
-            log_info("sql connection is down\n")
+            log_debug("sql connection is down\n")
 
         return None, -1
 
@@ -655,21 +683,20 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
     def get_server_variable(self, variable, default = None):
         return self.server_variables.get(variable, default)
 
-    #---------------------------------------------------------------------------
-    def open_ssh_session_for_monitoring(self):
-        ssh = SSH(self.server_profile, self.password_handler)
-
-        return ssh
-
     #---------------------------------------------------------------------------
     def is_ssh_connected(self):
-        return self.ssh is not None
-
+        return self.sshBridge and self.sshBridge.isConnected() == 1
 
     #---------------------------------------------------------------------------
     def query_server_info(self):
         self.server_variables = {}
         result = self.exec_query("SHOW VARIABLES")
+        
+        if not result:
+            # Didn't get the server variables. Assuming the server is stopped
+            self.last_known_server_running_status = ("stopped", time.time())
+            return
+        
         while result and result.nextRow():
             self.server_variables[result.stringByName("Variable_name")] = result.stringByName("Value")
 
@@ -678,15 +705,15 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
         result = self.exec_query("SHOW GLOBAL STATUS")
         while result and result.nextRow():
             self.status_variables[result.stringByName("Variable_name")] = result.stringByName("Value")
-
+            
         # check version
         if self.server_variables:
             self.raw_version = self.server_variables["version"]
             self.target_version = Version.fromstr(self.raw_version)
 
             if self.server_profile.server_version != self.raw_version: # Update profile version with live data from server
-                log_info('%s.connect_sql(): The server version stored in the server instance profile was "%s". '
-                            'Changed to the version reported by the server: "%s"\n' % (self.__class__.__name__,
+                log_debug('%s.connect_sql(): The server version stored in the server instance profile was "%s". '
+                            'Changed it to the version reported by the server: "%s"\n' % (self.__class__.__name__,
                             self.server_profile.server_version, self.raw_version) )
                 self.server_profile.server_version = self.raw_version
 
@@ -702,6 +729,11 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
                 plugin_type = result.stringByName("Type")
                 if status == "ACTIVE":
                     self.server_active_plugins.add((name, plugin_type))
+        if "offline_mode" in self.server_variables and self.server_variables["offline_mode"] == "ON":
+            #We're in offline mode, need to change server status
+            self.last_known_server_running_status = ("offline", time.time())
+        else:
+            self.last_known_server_running_status = ("running", time.time())
 
 
     def query_server_installation_info(self):
@@ -841,11 +873,11 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
             
             
             os_type: one of the main types of OS supported (one of wbaOS.windows, wbaOS.linux, wbaOS.darwin)
-            os_name: the exact name of the OS (eg Windows, Linux, Mac OS X, Solaris)
-            os_variant: the variant of the OS, esp for Linux distributions (eg Ubuntu, Fedora etc)
-            os_version: the version of the OS (eg 12.04, XP etc)
+            os_name: the exact name of the OS (for example Windows, Linux, macOS, Solaris)
+            os_variant: the variant of the OS, especially for Linux distributions (for example Ubuntu, Fedora etc.)
+            os_version: the version of the OS (for example 19.10)
         """
-        if self.ssh or self.server_profile.is_local:
+        if self.is_ssh_connected() or self.server_profile.is_local:
             o = StringIO.StringIO()
             # check if windows
             rc = self.server_helper.execute_command('ver', output_handler=o.write)
@@ -925,5 +957,3 @@ uses_ssh: %i uses_wmi: %i\n""" % (self.server_profile.uses_ssh, self.server_prof
                 else:
                     log_error("Could not execute uname command on remote server, system type is unknown\n")
         return None
-
-
